#ifndef _U3_H
#define _U3_H

#include <unistd.h>
#include <sys/time.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

typedef unsigned char uint8;
typedef unsigned short uint16;

//structure for storing calibration constants
struct U3_CALIBRATION_INFORMATION{
  double ainSESlope;
  double ainSEOffset;
  double ainDiffSlope;
  double ainDiffOffset;
  double dacSlope[2];
  double dacOffset[2];
  double tempSlope;
  double tempSlopeLow;
  double calTemp;
  double vref;
  double vref15;
  double vreg;
};

typedef struct U3_CALIBRATION_INFORMATION u3CalibrationInfo;

//functions
void normalChecksum(uint8 *b, int n);
void extendedChecksum(uint8 *b, int n);
uint8 normalChecksum8(uint8 *b, int n);
uint16 extendedChecksum16(uint8 *b, int n);
uint8 extendedChecksum8(uint8 *b);
int openUSBConnection(char *pathname);
int closeUSBConnection(int fd);
long getTickCount();
long binaryToCalibratedAnalogVoltage(u3CalibrationInfo *caliInfo, uint8 negChannel, uint16 bytesVoltage, double *analogVoltage);
long binaryToCalibratedAnalogTemperature(u3CalibrationInfo *caliInfo, uint16 bytesTemperature, double *analogTemperature);
long analogToCalibratedBinaryVoltage(u3CalibrationInfo *caliInfo, int DACNumber, double analogVoltage, uint8 *bytesVoltage);
long binaryToUncalibratedAnalogVoltage(uint8 gainBip, uint16 bytesVoltage, double *analogVoltage);
long analogToUncalibratedBinaryVoltage(double analogVoltage, uint8 *bytesVoltage);
double FPuint8ArrayToFPDouble(uint8 *buffer, int startIndex);
long getCalibrationInfo(int fd, u3CalibrationInfo *caliInfo);
  
#endif
