#ifndef _UE9_H
#define _UE9_H

#include <unistd.h>
#include <sys/time.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

typedef unsigned char uint8;
typedef unsigned short uint16;

//structure for storing calibration constants
struct CALIBRATION_INFORMATION{
  double unipolarSlope[4];
  double unipolarOffset[4];
  double bipolarSlope;
  double bipolarOffset;
  double DACSlope[2];
  double DACOffset[2];
  double tempSlope;
  double tempSlopeLow;
  double calTemp;
  double Vref;
  double VrefDiv2;
  double VsSlope;
};

typedef struct CALIBRATION_INFORMATION calibrationInfo;

//functions
void normalChecksum(uint8 *b, int n);
void extendedChecksum(uint8 *b, int n);
uint8 normalChecksum8(uint8 *b, int n);
uint16 extendedChecksum16(uint8 *b, int n);
uint8 extendedChecksum8(uint8 *b);
int openUSBConnection(char *pathname);
int closeUSBConnection(int fd);
long getTickCount();
long binaryToCalibratedAnalogVoltage(calibrationInfo *caliInfo, uint8 gainBip, uint16 bytesVoltage, double *analogVoltage);
long binaryToCalibratedAnalogTemperature(calibrationInfo *caliInfo, int powerLevel, uint16 bytesTemperature, double *analogTemperature);
long analogToCalibratedBinaryVoltage(calibrationInfo *caliInfo, int DACNumber, double analogVoltage, uint16 *bytesVoltage);
long binaryToUncalibratedAnalogVoltage(uint8 gainBip, uint16 bytesVoltage, double *analogVoltage);
long analogToUncalibratedBinaryVoltage(double analogVoltage, uint16 *bytesVoltage);
double FPuint8ArrayToFPDouble(uint8 *buffer, int startIndex);
long getCalibrationInfo(int fd, calibrationInfo *caliInfo);
  
#endif
