//Author: LabJack
//Oct. 19, 2005
//This example program calls the Feedback function.  DAC0 
//will be set to 2.5 volts and DAC1 will be set to 3.5 volts.  
//The states and directions will be read from FIO0 - FIO3 
//and the voltages (calibrated) from AI0-AI3.
#include "ue9.h"

union uShort{
  uint16 v;  
  uint8 c[2];
};
 
int feedback_example(int fd, calibrationInfo *caliInfo);

int main(int argc, char **argv)
{
  int fd;
  calibrationInfo caliInfo;

  /*
  if(argc < 2)
  {
    printf("Please enter a path name to the LabJack UE9 device.\n");
    exit(0);
  }
  else if(argc > 2)
  {
    printf("Too many arguments.\n");
    printf("Please enter a only a path name.\n");
    exit(0);
  }
  fd = openUSBConnection(argv[1]);
  */
  fd = openUSBConnection("/dev/usb/labjackue9_0");

  if(fd<0)
    return 1;

  if(getCalibrationInfo(fd, &caliInfo) < 0)
    return 1;
  
  feedback_example(fd, &caliInfo);

  if(closeUSBConnection(fd)< 0)
  {
    printf("Error: failed to close USB connection.\n");
    return 1;
  }

  return 0;
}


int feedback_example(int fd, calibrationInfo *caliInfo)
{
  uint8 sendBuff[34];
  uint8 recBuff[64];
  int sendChars;
  int recChars;
  double voltage;
  int i;
  unsigned int tempDir, tempState;
  union uShort readAI;
  uint16 checksumTotal;
  uint16 bytesVoltage;

  sendBuff[1] = (uint8)(0xF8);  //command byte
  sendBuff[2] = (uint8)(0x0E);  //number of data words
  sendBuff[3] = (uint8)(0x00);  //extended command number
  
  //all these bytes are set to zero since we are not changing
  //the FIO, EIO, CIO and MIO directions and states 
  for(i = 6; i <= 15; i++)
    sendBuff[i] = (uint8)(0x00);

  if(analogToCalibratedBinaryVoltage(caliInfo, 0, 2.500, &bytesVoltage) < 0)
    return -1;

  //setting the voltage of DAC0
  sendBuff[16] = (uint8)( bytesVoltage & (0x00FF) ); //low bits of DAC0
  sendBuff[17] = (uint8)( bytesVoltage >> 8 ) + 192; //high bits of DAC0 
                                                     //(bit 7 : Enable, bit 6: Update)
  if(analogToCalibratedBinaryVoltage(caliInfo, 1, 3.500, &bytesVoltage) < 0)
    return -1;
    
  //setting the voltage of DAC1
  sendBuff[18] = (uint8)( bytesVoltage & (0x00FF) ); //low bits of DAC1
  sendBuff[19] = (uint8)( bytesVoltage >> 8 ) + 192; //high bits of DAC1 
                                                     //(bit 7 : Enable, bit 6: Update)
  sendBuff[20] = (uint8)(0x0f);  //AINMask - reading AIN0 - AIN3, not AIN4 - AIN7
  sendBuff[21] = (uint8)(0x00);  //AINMask - not reading AIN8 - AIN15
  sendBuff[22] = (uint8)(0x00);  //AIN14ChannelNumber - not using
  sendBuff[23] = (uint8)(0x00);  //AIN15ChannelNumber - not using
  sendBuff[24] = (uint8)(0x0C);  //Resolution = 12

  //setting BipGains (Gain = 1, Bipolar = 0) 
  for(i = 25; i < 34; i++)
    sendBuff[i] = (uint8)(0x00);

  extendedChecksum(sendBuff, 34);
  
  /* Using Endpoint 2 to write/read.  Look at the labjackue9.c
   * (in the driver folder) functions labjackue9_read and 
   * labjackue9_write for details on using different endpoints.
   * Make note that count is the number of bytes to read/write. 
   */ 
  sendChars = write(fd, sendBuff, 34 + 32768);

  if(sendChars < 34)
  {
    if(sendChars == -1)
      printf("Error : write failed\n");
    else  
      printf("Error : did not write all of the buffer\n");
    return -1;
  }
  
  recChars = read(fd, recBuff, 64 + 32768);

  if(recChars < 64)
  {
    if(recChars == -1)
      printf("Error : read failed\n");
    else  
      printf("Error : did not read all of the buffer\n");
    return -1;
  }

  checksumTotal = extendedChecksum16(recBuff, 64);
  if( (uint8)((checksumTotal >> 8) & 0xff) != recBuff[5])
  {
    printf("Error : read buffer has bad checksum16(MSB)\n");
    return -1;
  }

  if( (uint8)(checksumTotal & 0xff) != recBuff[4])
  {
    printf("Error : read buffer has bad checksum16(LBS)\n");
    return -1;
  }

  if( extendedChecksum8(recBuff) != recBuff[0])
  {
    printf("Error : read buffer has bad checksum8\n");
    return -1;
  }

  if(recBuff[1] != (uint8)(0xF8) || recBuff[2] != (uint8)(0x1D) || recBuff[3] != (uint8)(0x00))
  {
    printf("Error : read buffer has wrong command bytes \n");
    return -1;
  }

  printf("Set DAC0 to 2.500 volts and DAC1 to 3.500 volts.\n\n");
  printf("Flexible digital I/O directions and states (FIO0 - FIO3):\n");
  for(i = 0; i < 4; i++)
  {
    tempDir = (recBuff[6] << (31 - i));
    tempDir = (tempDir >> 31);
    tempState = (recBuff[7] << (31 - i));
    tempState = (tempState >> 31);
    printf("  FI%d: %d and %d\n", i, tempDir, tempState);
  }

  printf("\nAnalog Inputs (AI0 - AI3):\n");
  for(i = 0; i < 4; i++)
  {
    readAI.c[0] = recBuff[12 + 2*i];
    readAI.c[1] = recBuff[13 + 2*i];
    bytesVoltage = readAI.v;

    //getting analog voltage
    if(binaryToCalibratedAnalogVoltage(caliInfo, (uint8)(0x00), bytesVoltage, &voltage) < 0)
      return -1;
    
    printf("  AI%d: %.4f V\n", i, voltage);
  }
  printf("\n");

  return 0;
}
