//Author: LabJack
//Oct. 19, 2005
//This example program makes 3 SingleIO calls.  One call sets DAC0 to
//2.500 V.  One call reads voltage from AIN0.  One call reads the 
//temperature from the internal temperature sensor.  Need control 
//firmware version 1.03 and above for the SingleIO function to work.
#include "ue9.h"

union uShort{
  uint16 v;  
  uint8 c[2];
};
 
int singleIO_AV_example(int fd, calibrationInfo *caliInfo);

int main(int argc, char **argv)
{
  int fd;
  calibrationInfo caliInfo;
  //int loop;
  //long start, total;

  /*
  if(argc < 2)
  {
    printf("Please enter a path name to the LabJack UE9 device.\n");
    exit(0);
  }
  else if(argc > 2)
  {
    printf("Too many arguments.\n");
    printf("Please enter a only a path name.\n");
    exit(0);
  }

  fd = openUSBConnection(argv[1]);
  */
  fd = openUSBConnection("/dev/usb/labjackue9_0");

  if(fd<0)
    return 1;

  if(getCalibrationInfo(fd, &caliInfo) < 0)
    return 1;

  /* 
  start = getTickCount();
  for(loop = 0; loop < 1000; loop++)
  { 
  */
  if(singleIO_AV_example(fd, &caliInfo) < 0)
  {
    closeUSBConnection(fd);
    return 1;
  }
  /*
  }

  total = getTickCount() - start;
  printf("1000 IOStream reads in %ld ms.\n", total);
  */
  if(closeUSBConnection(fd)< 0)
  {
    printf("Error: failed to close USB connection.\n");
    return 1;
  }

  return 0;
}


int singleIO_AV_example(int fd, calibrationInfo *caliInfo)
{
  uint8 sendBuff[8];
  uint8 recBuff[8];
  int sendChars, recChars;
  double voltage;
  double temperature; //in Kelvins
  union uShort readAIN;
  union uShort readTemperature;
  uint16 bytesVoltage;
  uint16 bytesTemperature;
  
  //////////////////////////////////// 
  //set voltage of DAC0 to 2.500 V
  ////////////////////////////////////
  
  //if(analogToUncalibratedBinaryVoltage(2.500, &bytesVoltage) < 0)  
  if(analogToCalibratedBinaryVoltage(caliInfo, 0, 2.500, &bytesVoltage) < 0) 
    return -1;
   
  sendBuff[1] = (uint8)(0xA3);  //command byte
  sendBuff[2] = (uint8)(0x05);  //IOType = 5 (analog out)
  sendBuff[3] = (uint8)(0x00);  //Channel = 0 (DAC0)
  sendBuff[4] = (uint8)( bytesVoltage & (0x00FF) );  //low bits of voltage
  sendBuff[5] = (uint8)( bytesVoltage >> 8 ) + 192;  //high bits of voltage 
                                                     //(bit 7 : Enable, bit 6: Update)
  sendBuff[6] = (uint8)(0x00);  //Settling time - does not apply to analog output
  sendBuff[7] = (uint8)(0x00);  //Reserved
  sendBuff[0] = normalChecksum8(sendBuff, 8);

  sendChars = write(fd, sendBuff, 8);
  if(sendChars < 8)
  {
    if(sendChars == -1)
      goto sendError0;
    else  
      goto sendError1;
  }
  
  recChars = read(fd, recBuff, 8);
  if(recChars < 8)
  {
    if(recChars == -1)
      goto recvError0;
    else  
      goto recvError1;
  }

  if((uint8)(normalChecksum8(recBuff, 8)) != recBuff[0])
    goto chksumError;

  if(recBuff[1] != (uint8)(0xA3))
    goto commandByteError;
  
  /*
  if(recBuff[2] != (uint8)(0x05))
    goto IOTypeError;
  
  if(recBuff[3] != 0)
    goto channelError;
  */
  
  printf("Set DAC0 voltage to 2.500 V ...\n");
    
  ///////////////////////////
  //read voltage from AIN0
  ///////////////////////////
  sendBuff[1] = (uint8)(0xA3);  //command byte
  sendBuff[2] = (uint8)(0x04);  //IOType = 4 (analog in)
  sendBuff[3] = (uint8)(0x00);  //Channel = 0 (AIN0)
  sendBuff[4] = (uint8)(0x00);  //BipGain (Bip = unipolar, Gain = 1)
  sendBuff[5] = (uint8)(0x0C);  //Resolution = 12
  sendBuff[6] = (uint8)(0x00);  //SettlingTime = 0
  sendBuff[7] = (uint8)(0x00);  //Reserved
  sendBuff[0] = normalChecksum8(sendBuff, 8);

  sendChars = write(fd, sendBuff, 8);
  if(sendChars < 8)
  {
    if(sendChars == -1)
      goto sendError0;
    else  
      goto sendError1;
  }
  
  recChars = read(fd, recBuff, 8);
  if(recChars < 8)
  {
    if(recChars == -1)
      goto recvError0;
    else  
      goto recvError1;
  }

  if((uint8)(normalChecksum8(recBuff, 8)) != recBuff[0])
    goto chksumError;

  if(recBuff[1] != (uint8)(0xA3))
    goto commandByteError;
    
  if(recBuff[2] != (uint8)(0x04))
    goto IOTypeError;
  
  if(recBuff[3] != 0)
    goto channelError;
  
  readAIN.c[0] = recBuff[5];
  readAIN.c[1] = recBuff[6];
  bytesVoltage = readAIN.v;

  //if(binaryToUncalibratedAnalogVoltage(sendBuff[4], bytesVoltage, &voltage) < 0)
  if(binaryToCalibratedAnalogVoltage(caliInfo, sendBuff[4], bytesVoltage, &voltage) < 0) 
    return -1;

  printf("Voltage read from AI0: %.4f V\n", voltage);
    
  ///////////////////////////////////////////////////
  //read temperature from internal temperature sensor 
  ///////////////////////////////////////////////////
  sendBuff[1] = (uint8)(0xA3);  //command byte
  sendBuff[2] = (uint8)(0x04);  //IOType = 4 (analog in)
  sendBuff[3] = (uint8)(0x85);  //Channel = 133 (tempSensor)
  sendBuff[4] = (uint8)(0x00);  //Gain = 1 (Bip does not apply)
  sendBuff[5] = (uint8)(0x0C);  //Resolution = 12
  sendBuff[6] = (uint8)(0x00);  //SettlingTime = 0
  sendBuff[7] = (uint8)(0x00);  //Reserved
  sendBuff[0] = normalChecksum8(sendBuff, 8);

  sendChars = write(fd, sendBuff, 8);
  if(sendChars < 8)
  {
    if(sendChars == -1)
      goto sendError0;
    else  
      goto sendError1;
  }
  
  recChars = read(fd, recBuff, 8);
  if(recChars < 8)
  {
    if(recChars == -1)
      goto recvError0;
    else  
      goto recvError1;
  }

  if((uint8)(normalChecksum8(recBuff, 8)) != recBuff[0])
    goto chksumError;

  if(recBuff[1] != (uint8)(0xA3))
    goto commandByteError;
    
  if(recBuff[2] != (uint8)(0x04))
    goto IOTypeError;

  if(recBuff[3] != (uint8)(0x85))
    goto channelError;
        
  readTemperature.c[0] = recBuff[5];
  readTemperature.c[1] = recBuff[6];
  bytesTemperature = readTemperature.v;

  //assuming high power level
  if(binaryToCalibratedAnalogTemperature(caliInfo, 0, bytesTemperature, &temperature) < 0)
    return -1;
  
  printf("Temperature read internal temperature sensor (channel 133): %.1f K\n\n", temperature);
  return 0;
    
//error printouts
sendError0:
  printf("Error : write failed\n");
  return -1;
sendError1:
  printf("Error : did not write all of the buffer\n");
  return -1;
recvError0:
  printf("Error : read failed\n");
  return -1;
recvError1:  
  printf("Error : did not read all of the buffer\n");
  return -1;
chksumError:
  printf("Error : read buffer has bad checksum\n");
  return -1;
commandByteError:
  printf("Error : read buffer has wrong command byte\n");
  return -1;
IOTypeError:  
  printf("Error : read buffer has wrong IOType\n");
  return -1;
channelError:  
  printf("Error : read buffer has wrong channel\n");
  return -1;
}
