//Author: LabJack
//March 22, 2005
//This example program reads analog inputs AI0 - AI3 using stream mode
#include "ue9.h"

union uShort{
  uint16 v;  
  uint8 c[2];
};

int flushStream(int fd);
int StreamConfig_example(int fd);
int StreamStart(int fd);
int StreamData_example(int fd, calibrationInfo *caliInfo);
int StreamStop(int fd);

int main(int argc, char **argv)
{
  int fd;
  calibrationInfo caliInfo;

  /*
  if(argc < 2)
  {
    printf("Please enter a path name to the LabJack UE9 device.\n");
    exit(0);
  }
  else if(argc > 2)
  {
    printf("Too many arguments.\n");
    printf("Please enter a only a path name.\n");
    exit(0);
  }
  fd = openUSBConnection(argv[1]);
  */
  fd = openUSBConnection("/dev/usb/labjackue9_0");

  if(fd<0)
    return 1;

  flushStream(fd);
  
  if(getCalibrationInfo(fd, &caliInfo) < 0)
    return 1;

  if(StreamConfig_example(fd) != 0)
    return 1;

  if(StreamStart(fd) != 0)
    return 1;

  StreamData_example(fd, &caliInfo);
  StreamStop(fd);

  if(closeUSBConnection(fd)< 0)
  {
    printf("Error: failed to close USB connection.\n");
    return 1;
  }

  return 0;
}

//stream is configured for reading only 4 analog inputs (AI0 - AI1)
int StreamConfig_example(int fd)
{
  uint8 sendBuff[20];
  uint8 recBuff[8];
  int sendChars, recChars;
  uint16 checksumTotal;

  sendBuff[1] = (uint8)(0xF8);  //command byte
  sendBuff[2] = (uint8)(0x07);  //number of data words : NumChannels + 3 (NumChannels = 4)
  sendBuff[3] = (uint8)(0x11);  //extended command number
  sendBuff[6] = (uint8)(0x04);  //NumChannels = 4  
  sendBuff[7] = (uint8)(0x0C);  //resolution = 12
  sendBuff[8] = (uint8)(0x00);  //SettlingTime = 0
  sendBuff[9] = (uint8)(0x00);  //ScanConfig: scan pulse and external scan trigger disabled
                                //stream clock frequency = 4 MHz
  //scan interval = 4000
  sendBuff[10] = (uint8)(0xA0); //scan interval (low byte)
  sendBuff[11] = (uint8)(0x0F);	//scan interval (high byte)
  sendBuff[12] = (uint8)(0x00); //channel # = 0
  sendBuff[13] = (uint8)(0x00); //BipGain (Bip = unipolar, Gain = 1)
  sendBuff[14] = (uint8)(0x01); //channel # = 1
  sendBuff[15] = (uint8)(0x00); //BipGain (Bip = unipolar, Gain = 1)
  sendBuff[16] = (uint8)(0x02); //channel # = 2
  sendBuff[17] = (uint8)(0x00); //BipGain (Bip = unipolar, Gain = 1)
  sendBuff[18] = (uint8)(0x03); //channel # = 3
  sendBuff[19] = (uint8)(0x00); //BipGain (Bip = unipolar, Gain = 1)

  extendedChecksum(sendBuff,20);
  
  sendChars = write(fd, sendBuff, 20);
  if(sendChars < 20)
  {
    if(sendChars == -1)
      printf("Error : write failed (StreamConfig).\n");
    else  
      printf("Error : did not write all of the buffer (StreamConfig).\n");
    return -1;
  } 

  recChars = read(fd, recBuff, 8);

  if(recChars < 8)
  {
    if(recChars == -1)
      printf("Error : read failed (StreamConfig).\n");
    else  
      printf("Error : did not read all of the buffer (StreamConfig).\n");
    return -1;
  }

  checksumTotal = extendedChecksum16(recBuff, 8);
  if( (uint8)((checksumTotal >> 8) & 0xff) != recBuff[5])
  {
    printf("Error : read buffer has bad checksum16(MSB) (StreamConfig).\n");
    return -1;
  }

  if( (uint8)(checksumTotal & 0xff) != recBuff[4])
  {
    printf("Error : read buffer has bad checksum16(LBS) (StreamConfig).\n");
    return -1;
  }

  if( extendedChecksum8(recBuff) != recBuff[0])
  {
    printf("Error : read buffer has bad checksum8 (StreamConfig).\n");
    return -1;
  }

  if( recBuff[1] != (uint8)(0xF8) || recBuff[2] != (uint8)(0x01) || recBuff[3] != (uint8)(0x11) || recBuff[7] != (uint8)(0x00))
  {
    printf("Error : read buffer has wrong command bytes (StreamConfig).\n");
    return -1;
  }

  if(recBuff[6] != 0)
  {
    printf("Errorcode # %d from StreamConfig read.\n", (unsigned int)recBuff[6]);
    return -1;
  }

  return 0;
}

int StreamStart(int fd)
{
  uint8 sendBuff[2];
  uint8 recBuff[4];
  int sendChars, recChars;

  sendBuff[0] = (uint8)(0xA8);  //CheckSum8
  sendBuff[1] = (uint8)(0xA8);  //command byte
  
  sendChars = write(fd, sendBuff, 2);
  if(sendChars < 2)
  {
    if(sendChars == -1)
      printf("Error : write failed.\n");
    else  
      printf("Error : did not write all of the buffer.\n");
    return -1;
  } 

  recChars = read(fd, recBuff, 4);

  if(recChars < 4)
  {
    if(recChars == -1)
      printf("Error : read failed.\n");
    else  
      printf("Error : did not read all of the buffer.\n");
    return -1;
  }

  if( recBuff[1] != (uint8)(0xA9) || recBuff[3] != (uint8)(0x00) )
  {
    printf("Error : read buffer has wrong command bytes \n");
    return -1;
  }

  if(recBuff[2] != 0)
  {
	printf("Errorcode # %d from StreamStart read.\n", (unsigned int)recBuff[2]);
    return -1;
  }

  return 0;
}

//The ue9 streamData function is called 125*timesDisplayed times.
//All voltages from the sream are saved in the voltages 2D array
int StreamData_example(int fd, calibrationInfo *caliInfo)
{
  uint8 recBuff[192];
  int recChars;
  int i, j, k, m, packetCounter, currChannel, scanNumber, totalPackets;
  uint16 checksumTotal;
  union uShort readAI;
  double voltages[2480][4];  //each packet contains 4 samples for each channel (4 channels)
                             //4*125*(timesDisplayed = 5) 
  packetCounter = 0;
  currChannel = 0;
  scanNumber = 0;
  totalPackets = 0;
  recChars = 0;

  printf("Reading Samples...\n");

  for (i = 0; i < 5; i++)
  {
    for(j = 0; j < 31; j++)		//reading 124 packets of 192 bytes
    {
      /* For usb read 192 bytes instead of the 46.  The 192 bytes 
       * response is 4 StreamData packet responses of 48 bytes.
       */
      /* Using Endpoint 2 to read.  Look at the labjackue9.c
       * (in the driver folder) function labjackue9_read for 
       * details on using different endpoints.  Make note that 
       * count is the number of bytes to read. 
       */
      recChars = read(fd, recBuff, 192 + 32768);
	
      if(recChars < 192)
      {
        if(recChars == -1)
          printf("Error : read failed (StreamData).\n");
        else  
          printf("Error : did not read all of the buffer %d (StreamData).\n", recChars);

        return -1;
      }

      for (m = 0; m < 4; m++)
      { 
        totalPackets++;

        checksumTotal = extendedChecksum16(recBuff + m*48, 46);
        if( (uint8)((checksumTotal >> 8) & 0xff) != recBuff[m*48 + 5])
        {
          printf("Error : read buffer has bad checksum16(MSB) (StreamData).\n");
          return -1;
        }

        if( (uint8)(checksumTotal & 0xff) != recBuff[m*48 + 4])
        {
          printf("Error : read buffer has bad checksum16(LBS) (StreamData).\n");
          return -1;
        }

        checksumTotal = extendedChecksum8(recBuff + m*48);
        if( checksumTotal != recBuff[m*48])
        {
          printf("Error : read buffer has bad checksum8 (StreamData).\n");
          return -1;
        }

        if( recBuff[m*48 + 1] != (uint8)(0xF9) || recBuff[m*48 + 2] != (uint8)(0x14) || recBuff[m*48 + 3] != (uint8)(0xC0) )
        {
          printf("Error : read buffer has wrong command bytes (StreamData).\n");
          return -1;
        }

        if(recBuff[m*48 + 11] != 0)
        {
          printf("Errorcode # %d from StreamData read.\n", (unsigned int)recBuff[11]);
          return -1;
        }

        if(packetCounter != (int) recBuff[m*48 + 10])
        {
          printf("PacketCounter does not match with with current packet count (StreamData).\n");
          return -1;
        }
	
        //this check is only for usb stream reads since there are two extra bytes
        //added to the StreamData packet
	if(( recBuff[m*48 + 46] != (uint8)(0x00) ) && ( recBuff[m*48 + 47] != (uint8)(0x00) ))
        {
          printf("Error: The last two bytes of the StreamData packet are not 0x00.\n");
          return -1;
        }
 
        for(k = 12; k <43; k+=2)
        {
          readAI.c[0] = recBuff[m*48 + k];
          readAI.c[1] = recBuff[m*48 + k+1];
          binaryToCalibratedAnalogVoltage(caliInfo, (uint8)(0x00), readAI.v, &(voltages[scanNumber][currChannel])); 
          currChannel++;
          if(currChannel > 3)
          {
            currChannel = 0;
            scanNumber++;
          }
        }

        if(packetCounter >= 255)
          packetCounter = 0;
        else
          packetCounter++;
      }
    }

    printf("\nNumber of scans: %d\n", scanNumber);
    printf("Total packets read: %d\n", totalPackets);
    printf("Current PacketCounter: %d\n", (int)recBuff[154]);	
	  
    for(k = 0; k < 4; k++)
      printf("  AI%d: %.4f V\n", k, voltages[scanNumber - 1][k]);  
  }
  return 0;
}

int StreamStop(int fd)
{
  uint8 sendBuff[2];
  uint8 recBuff[4];
  int sendChars, recChars;

  sendBuff[0] = (uint8)(0xB0);  //CheckSum8
  sendBuff[1] = (uint8)(0xB0);  //command byte
  
  sendChars = write(fd, sendBuff, 2);
  if(sendChars < 2)
  {
    if(sendChars == -1)
      printf("Error : write failed (StreamStop).\n");
    else  
      printf("Error : did not write all of the buffer (StreamStop).\n");
    return -1;
  } 

  recChars = read(fd, recBuff, 4);

  if(recChars < 4)
  {
    if(recChars == -1)
      printf("Error : read failed (StreamStop).\n");
    else  
      printf("Error : did not read all of the buffer (StreamStop).\n");
    return -1;
  }

  if( recBuff[1] != (uint8)(0xB1) || recBuff[3] != (uint8)(0x00) )
  {
    printf("Error : read buffer has wrong command bytes (StreamStop).\n");
    return -1;
  }

  if(recBuff[2] != 0)
  {
    printf("Errorcode # %d from StreamStop read.\n", (unsigned int)recBuff[2]);
    return -1;
  }

  return 0;
}

int flushStream(int fd)
{
  uint8 sendBuff[2];
  uint8 recBuff[2];
  int sendChars, recChars;

  sendBuff[0] = (uint8)(0x08);  //CheckSum8
  sendBuff[1] = (uint8)(0x08);  //command byte

  sendChars = write(fd, sendBuff, 2);
  if(sendChars < 2)
  {
    if(sendChars == -1)
      printf("Error : write failed (flushStream).\n");
    else  
      printf("Error : did not write all of the buffer (flushStream).\n");
    return -1;
  } 

  recChars = read(fd, recBuff, 2);

  if(recChars < 2)
  {
    if(recChars == -1)
      printf("Error : read failed (flushStream).\n");
    else  
      printf("Error : did not read all of the buffer (flushStream).\n");
    return -1;
  }
   
  if(recBuff[0] != (uint8)(0x08) || recBuff[1] != (uint8)(0x08))
  {
    printf("Error : read buffer has wrong command bytes (flushStream).\n");
    return -1;
  }    
  return 0;  
}
